<!DOCTYPE html>

<%@page import="java.lang.management.MemoryType"%>
<%@page import="java.lang.management.MemoryPoolMXBean"%>
<%@page import="java.lang.management.MemoryUsage"%>
<%@page import="java.io.InputStreamReader"%>
<%@page import="java.io.BufferedWriter"%>
<%@page import="java.io.BufferedReader"%>
<%@page import="java.io.IOException"%>
<%@page import="java.io.FileOutputStream"%>
<%@page import="java.io.InputStream"%>
<%@page import="java.io.FileWriter"%>
<%@page import="java.io.OutputStream"%>
<%@page import="java.io.FileInputStream"%>
<%@page import="java.io.File"%>
<%@page import="java.lang.management.ManagementFactory"%>
<%@page import="java.lang.management.RuntimeMXBean"%>
<%@page import="java.util.List"%>

<%!
protected String[] getDatabaseDrivePackage(String database){
	String NO_DATABASE =  "No database";
	String ORACLE =  "Oracle";
	String DB2 =  "DB2";
	String MYSQL =  "MySQL";
	String POSTGRESQL =  "PostgreSQL";
	String DERBY =  "Derby";
	String SQL_SERVER =  "SQL Server";
	String SYBASE =  "Sybase";

	if(ORACLE.equals(database)){
		return new String[]{"oracle.jdbc"};
	} else if(DB2.equals(database)){
		return new String[]{"COM.ibm.db2.jdbc.app", "com.ibm.db2.jcc"};
	} else if(MYSQL.equals(database)){
		return new String[]{"com.mysql.jdbc"};
	} else if(POSTGRESQL.equals(database)){
		return new String[]{"org.postgresql"};
	} else if(DERBY.equals(database)){
		return new String[]{"org.apache.derby"};
	} else if(SQL_SERVER.equals(database)){
		return new String[]{"com.microsoft.sqlserver.jdbc", "net.sourceforge.jtds.jdbc"};
	} else if(SYBASE.equals(database)){
		return new String[]{"com.sybase.jdbc2.jdbc"};
	}
	return new String[0];
}

//To exclude capturing the method calls to jdbc drivers
//<pointcut name="excludedPointcut" expression="execution(* com.mysql.jdbc..*.*(..)) || execution(com.mysql.jdbc..*.new(..))"/>
protected String getAOPXmlExcludesPointcut(String databaseType){
	if(databaseType == null || databaseType.trim().length() == 0){
		return "";
	}
	String[] driverPackage = getDatabaseDrivePackage(databaseType);
	String executions = "expression=\"";
	for(int i=0; i<driverPackage.length; i++){
		executions = executions + "execution(* " + driverPackage[i] 
		                 + "..*.*(..)) || execution(" + driverPackage[i] + "..*.new(..))";
		if(i < driverPackage.length-1){
			executions = executions + " || ";
		}
	}
	return "<pointcut name=\"excludedPointcut\" " + executions +  "\"/>";
	//return "<pointcut name=\"excludedPointcut\" expression=\"execution(* " + driverPackage + "..*.*(..)) || execution(" +driverPackage + "..*.new(..))\"/>";  				
}

boolean generateAOPXML(String serverType, String incpackages, String excpackages, String databaseType, 
		 boolean verbose, boolean doNotLogData, String aopFolder) throws Exception{
	String[] dbDriverPackages = doNotLogData ? new String[0] : getDatabaseDrivePackage(databaseType);
	String[] incpackagesArr = incpackages.split(",");
	String[] excpackagesArr = new String[0];
	if(excpackages != null && excpackages.trim().length() > 0){
		excpackagesArr = excpackages.split(",");
	}

	String[] excpackagesSystem=new String[]{"*ObjectFactory", "*..*CGLIB*"};
	
	String aspectName = doNotLogData ? "com.maintainj.aspect.ExecutionAspect":"com.maintainj.aspect.ExecutionDataAspect";
	String aop="<aspectj>\n";
	aop+=		"\t<aspects>\n";
	aop+=			"\t\t<concrete-aspect name=\"com.maintainj.inst.J2EEAspect\" extends=\"" + aspectName + "\">\n";
	aop+=				"\t\t\t<pointcut name=\"anyExecution\" expression=\"execution(* *.*(..)) || execution(*.new(..))\"/>\n";
	if(!doNotLogData){//No need to exclude jdbc driver calls if context is not logged
	aop+=				"\t\t\t" + getAOPXmlExcludesPointcut(databaseType) + "\n";
	}
	aop+=			"\t\t</concrete-aspect>\n";
	aop+=		"\t</aspects>\n";
	
	
	String verboseString = verbose ? " -showWeaveInfo -verbose":"";
	aop+=		"\t<weaver options=\"-proceedOnError -nowarn -XaddSerialVersionUID  -Xset:makeTjpFieldsTransient=true,avoidFinal=true" + verboseString + "\">\n";

	for	(int index = 0; index < incpackagesArr.length; index++) {
		aop+=		"\t\t<include within=\""+incpackagesArr[index] + "..*\"/>\n";
	}
	
	for(int index = 0; index < dbDriverPackages.length; index++) {
		aop+=		"\t\t<include within=\""+dbDriverPackages[index] + "..*\"/>\n";
	}
	
	String[] jspPackNames = getJSPPackageName(serverType);
	for	(int index = 0; index < jspPackNames.length; index++) {
		aop+=		"\t\t<include within=\""+jspPackNames[index] + "..*\"/>\n";
	}

	for	(int index = 0; index < excpackagesArr.length; index++) {
	aop+=		"\t\t<exclude within=\""+excpackagesArr[index] + "..*\"/>\n";
	}

	for	(int index = 0; index < excpackagesSystem.length; index++) {
		aop+=		"\t\t<exclude within=\""+excpackagesSystem[index] + "\"/>\n";
	}

	aop+=		"\t</weaver>\n";
	
	aop+=	"</aspectj>";
	
	//Create the directory for aop.xml	        
    new File(aopFolder).mkdirs();
    
    String aopXmlFile = aopFolder + File.separator + "maintainj.xml";
    FileWriter fw = new FileWriter(aopXmlFile);
    fw.write(aop);
    fw.close();
    return true;
}

String[] getJSPPackageName(String serverType){
	//JSP package names
	String TOMCAT_JBOSS_JSP_PACKAGE_NAME = "org.apache.jsp";
	String WEBSPHERE_JSP_PACKAGE_NAME = "com.ibm._jsp";
	String WEBLOGIC_JSP_PACKAGE_NAME = "jsp_servlet";
	String[] jspPackNames = new String[0];
	if(serverType.equals("websphere")){
		jspPackNames = new String[]{WEBSPHERE_JSP_PACKAGE_NAME};
	} else if(serverType.equals("weblogic")){
		jspPackNames = new String[]{WEBLOGIC_JSP_PACKAGE_NAME};
	} else {
		jspPackNames = new String[]{TOMCAT_JBOSS_JSP_PACKAGE_NAME};
	}
	return jspPackNames;
}

void copyAgentJar(ServletContext context, String aopFolder) throws Exception{
	//Copy MaintainJAgent.jar to the same folder       
	String agentJar = "/MaintainJAgent.jar";	
	InputStream in = context.getResourceAsStream(agentJar);	
	OutputStream out = new FileOutputStream(new File(aopFolder + agentJar));
	int bufferSize = 16 * 1024;
	byte[] buffer = new byte[bufferSize];
	int bytesRead;
	while ((bytesRead = in.read(buffer)) >= 0) {			
		out.write(buffer, 0, bytesRead);
	}
	out.close();
	in.close();				
}

String replace(String oriString, String toReplace, String replaceWith){
	StringBuffer oriStringBuffer = new StringBuffer(oriString);
	while(oriStringBuffer.indexOf(toReplace) != -1){
		int index = oriStringBuffer.indexOf(toReplace);
		oriStringBuffer = oriStringBuffer.delete(index, index + toReplace.length());				
		oriStringBuffer = oriStringBuffer.insert(index, replaceWith);
	}
	return oriStringBuffer.toString();
}

void replaceStrings(InputStream fileStream, String newFile, String[] replace, String[] replaceWith) throws IOException{
	BufferedReader br = new BufferedReader(new InputStreamReader(fileStream));
	BufferedWriter bw = new BufferedWriter(new FileWriter(newFile));
	String line = null;
	while((line = br.readLine()) != null){
		for(int i=0;i<replace.length; i++){				
			line = replace(line, replace[i], replaceWith[i]);			
		}
		bw.write(line);			
		bw.newLine();			
	}
	br.close();
	bw.close();
}

String getScriptFilename(String serverType){
	String fileName = "";
	if("tomcat".equals(serverType)){
		if('/' == File.separatorChar){
			fileName = "/catalina_with_mnj.sh";
		} else {
			fileName = "/catalina_with_mnj.bat";
		}		
	} else if("jboss".equals(serverType)){
		if('/' == File.separatorChar){
			fileName = "/run_with_mnj.sh";
		} else {
			fileName = "/run_with_mnj.bat";
		}				
	} else if("wildfly".equals(serverType)){				
		if('/' == File.separatorChar){
			fileName = "/run_with_mnj.sh";
		} else {
			fileName = "/run_with_mnj.bat";
		}
	} else if("weblogic".equals(serverType)){		
		if('/' == File.separatorChar){
			fileName = "/startWebLogic_with_mnj.sh";
		} else {
			fileName = "/startWebLogic_with_mnj.cmd";
		}
	}
	return fileName;
}

MemoryUsage getPermGenMemoryUsage(){
	List mpBeans = ManagementFactory.getMemoryPoolMXBeans();
	for(int i=0; i<mpBeans.size(); i++){
		MemoryPoolMXBean mpBean = (MemoryPoolMXBean)mpBeans.get(i);
		if (mpBean.getType() != MemoryType.HEAP) {			
			if(mpBean.getName().contains("Perm Gen")){
				return mpBean.getUsage();	
			}
		}
	}
	return null;
}

boolean isPermGenTooLow(){
	boolean isPermGenTooLow = false;
	MemoryUsage mu = getPermGenMemoryUsage();
	if(mu != null){
		if(mu.getMax() < mu.getUsed() * 1.5){
			isPermGenTooLow = true;
		}
	}
	return isPermGenTooLow;
}

boolean copyScriptFile(ServletContext application, String serverType, String aopFolder, String scriptFileName) throws IOException{
	boolean scriptFileCopied = false;
	if(scriptFileName != null && scriptFileName.length() > 0){
		try{
			InputStream is = application.getResourceAsStream("/install/" + serverType + scriptFileName);	
			if(is == null){
				is = getClass().getResourceAsStream("/install/" + serverType + scriptFileName);
			}
		
			replaceStrings(is, aopFolder + File.separator + scriptFileName, 
					new String[]{"@INSTALL_HOME@"}, 
			    	new String[]{aopFolder});
			scriptFileCopied = true;
		} catch (Exception e){				
			e.printStackTrace();
		}				
	}
	return scriptFileCopied;
}

/**
* To build the message to show to the user
*/
String getMessage(boolean aopGenerationSuccess, boolean copyAgentSuccess, boolean installScriptCopySuccess, 
		String folderSelected, String fileName, String serverType){
	String result = "";
	if(aopGenerationSuccess && copyAgentSuccess){
		result = "<span class=\"alert alert-success\">Success! See below for the next step.</span>"
			+ "<br>i) MaintainJAgent.jar is placed in " + folderSelected + ". <br>ii) The configuration file maintainj.xml is generated and placed in the same folder. ";	
	} else if (aopGenerationSuccess && !copyAgentSuccess){
		result = "<span class=\"alert alert-success\">Success! See below for the next step.</span>"
				+ "<br>i) maintainj.xml is generated and placed in the same folder. <br>ii) Please take the MaintainJAgent.jar from MaintainJ.war and place it in " + folderSelected + ". <br>";
	} else if(!aopGenerationSuccess){
		result = "<span class=\"alert alert-danger\">Something is wrong. Please email to support@maintainj.com with any exception stack traces in System.out.</span>";			
	}

	if("websphere".equals(serverType)){
		String sep = File.separator;
		result = result + "<p>Restart WebSphere after performing two changes below:<br>"
				+" i) Add the below two JVM arguments to server. " 
				+ "You can add the JVM arguments in WebSphere admin console by following the menu sequece similar to "
				+ "Servers->Application Servers->server1->Java and Process management->Process Definition->Java Virtual Machine";
		result = result + "<br>Append the below line to the 'Generic JVM arguments' field<br><code>";
		result = result + "-javaagent:" + folderSelected + File.separator + "MaintainJAgent.jar";
		result = result + " -Dorg.aspectj.weaver.loadtime.configuration=file:" + folderSelected + File.separator + "maintainj.xml</code>";
		
		result = result +"<br>ii) Append the below permissions to the existing permissions in the java policy file at " 
				+ System.getProperty("java.home") + sep + "lib" + sep + "security" + sep + "java.policy."
				+ " These permissions are required for MaintainJ's agent to work in WebSphere.";
		result = result +"<code><br>permission java.util.PropertyPermission \"aj.weaving.verbose\", \"read\";"; 
		result = result +"<br>permission java.util.PropertyPermission \"org.aspectj.tracing.messages\", \"read\";";  
		result = result +"<br>permission java.util.PropertyPermission \"org.aspectj.weaver.showWeaveInfo\", \"read\";";  
		result = result +"<br>permission java.util.PropertyPermission \"org.aspectj.weaver.loadtime.configuration\", \"read\";</code>";	

		  
	} else if(aopGenerationSuccess && installScriptCopySuccess){
		result = result + "<br>iii) Server startup script file <mark>" + fileName.substring(1) + "</mark> is placed in " + folderSelected + "." ;	
		if("tomcat".equals(serverType)){
			result = result +" Copy this file to tomcat bin folder where catalina." + getBatSh() +" file exists.";
		} else if("jboss".equals(serverType)){
			result = result + " Copy this file to JBoss bin folder where run." +  getBatSh() + " file exists.";
		} else if("weblogic".equals(serverType)){
			result = result +" Copy this file to WebLogic domain folder where startWebLogic." + getCmdSh() + " file exists.";
		} else if("wildfly".equals(serverType)){
			result = result +" Copy this file to Wildfly or JBoss EAP bin folder where standalone." + getBatSh() +" file exists.";
		}

		result = result + " Restart the server using this script before verifying the installation in Step 2."
				+ " If you don't use the script to start server or if this script fails to restart the server or if the verification fails, "
				+ "use " + fileName.substring(1) + " as a guideline to add the JVM arguments to the server."			
				+ " For most application servers, the below two JVM arguments need to be added:<br>";
		result = result + "  <code>-javaagent:" + folderSelected + File.separator + "MaintainJAgent.jar</code><br>";
		result = result + "  <code>-Dorg.aspectj.weaver.loadtime.configuration=file:" + folderSelected + File.separator + "maintainj.xml</code>";
	} else {
		result = result + "<br>Please add the below two JVM arguments to server and restart the server.<br>";
		result = result + "  <code>-javaagent:" + folderSelected + File.separator + "MaintainJAgent.jar</code><br>";
		result = result + "  <code>-Dorg.aspectj.weaver.loadtime.configuration=file:" + folderSelected + File.separator + "maintainj.xml</code>";
	}
	
	if(isPermGenTooLow()){
		MemoryUsage permMU = getPermGenMemoryUsage();
		result = result + "<br> iv) MaintainJ needs about 50-60% more Perm Gen space than what your application normally needs, "
				+ "if you trace all the classes in your application. "
				+ "The current maximum Perm Gen is " + (permMU.getMax()/(1024*1024)) + "MB and "
				+ "the application is already using " + (permMU.getUsed()/(1024*1024)) + "MB. " 
				+ "If your application is very slow to start with MaintainJ or if you run into "
				+ "OutOfMemoryError when you start the server with MaintainJ, "
				+ "please increase the maximum possible Perm Gen space for the server.<br> "
				+ "Example to set this JVM argument to 512MB: <code>-XX:MaxPermSize=512m</code>";
	}
	return result;
}

String getVerifyFailedMessage(boolean maintainjInCP, boolean agentRegistered){
	RuntimeMXBean runtimemxbean = ManagementFactory.getRuntimeMXBean();
    List list = runtimemxbean.getInputArguments();
    String[] jvmArgs = new String[0];
    if(list.size() > 0){
    	jvmArgs = (String[])list.toArray(new String[list.size()]);        			
    }
    String errorMessage = (maintainjInCP ? "" : "MaintainJAgent.jar does not seem to be in classpath. ");
    errorMessage = errorMessage + (agentRegistered ? "" : 
    	"<br>MaintainJ agent is not registered. Check if -javaagent is correctly set to MaintainJAgent.jar.");
    
	String result = "<span class=\"alert alert-danger\">Installation failed.</span><br>" 
		+ errorMessage
		+ "<br>Current JVM arguments are:<br><code>";
	for(int j=0; j<jvmArgs.length; j++){									
		result = result + jvmArgs[j]+"<br>";
	}
	result = result + "</code>";
	result = result + "Current System classpath:<br><code>" + System.getProperty("java.class.path");
	result = result + "</code><br>Email to support@maintainj.com with the above error message and server logs if you are unable to resolve the problem quickly.";
	return result;
}

void appendToFile() throws Exception{
	String sep = File.separator;
	String fileName = System.getProperty("java.home") + sep + "lib" + sep + "security" + sep + "java.policy";
	
    BufferedWriter bufferWritter = new BufferedWriter(new FileWriter(fileName,true));
    
    bufferWritter.write("permission java.util.PropertyPermission \"aj.weaving.verbose\", \"read\";");
    bufferWritter.newLine();
    bufferWritter.write("permission java.util.PropertyPermission \"org.aspectj.tracing.messages\", \"read\";");
    bufferWritter.newLine();
    bufferWritter.write("permission java.util.PropertyPermission \"org.aspectj.weaver.showWeaveInfo\", \"read\";");
    bufferWritter.newLine();
    bufferWritter.write("permission java.util.PropertyPermission \"org.aspectj.weaver.loadtime.configuration\", \"read\";");
    bufferWritter.newLine();
    
    bufferWritter.close();
}

String getBatSh(){
	return (File.separatorChar == '/' ? "sh" : "bat");
}

String getCmdSh(){
	return (File.separatorChar == '/' ? "sh" : "cmd");
}
%>

<%
String aopFolder = null;
String jvmArgs[] = new String[]{"There are no JVM arguments"};
String nextAction=request.getParameter("nextAction");
boolean installationSuccess = false;	
if("verify".equals(nextAction)){	
	boolean mnjJarInCP = true;
	try{
		Class.forName("com.maintainj.aspect.JoinPointMap");
	} catch (Throwable th){
		mnjJarInCP = false;
		th.printStackTrace();
	}	
	boolean aspectRegistered = true;
	try{	
		Class.forName("com.maintainj.inst.J2EEAspect");
	} catch (Throwable th){
		aspectRegistered = false;
		th.printStackTrace();
	}			
	installationSuccess = (mnjJarInCP && aspectRegistered);			        
    if(!installationSuccess){	        
	   	out.println(getVerifyFailedMessage(mnjJarInCP, aspectRegistered));
    } else {
      	out.println("<span class=\"alert alert-success\">MaintainJ is installed and configured successfully!</span>");
		out.println("<a href=\"trace.jsp\">Click to start tracing the application</a>.");
    }
	return;
} else if("downloadAopXml".equals(nextAction)){
	String serverType = request.getParameter("serverType");
	String incpackages = request.getParameter("incpackages");
	String excpackages = request.getParameter("excpackages");
	String databaseType = request.getParameter("databaseType");
	boolean verbose = "true".equals(request.getParameter("verbose"));
	boolean doNotlogData = "true".equals(request.getParameter("logData"));
	aopFolder = request.getParameter("aopFolder");
	boolean aopGenerationSuccess = false;
	try{
		generateAOPXML(serverType, incpackages, excpackages, databaseType, verbose, doNotlogData, aopFolder);
		aopGenerationSuccess = true;
	} catch (Exception e){
		e.printStackTrace();
	}
	
	boolean copyAgentSuccess = false;
	if(aopGenerationSuccess){
		//Copy MaintainJAgent.jar to the same folder
		try{
			copyAgentJar(application, aopFolder);
			copyAgentSuccess = true;
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	//Copy the install script file			
	String scriptFileName = getScriptFilename(serverType);
	boolean scriptFileCopied = copyScriptFile(application, serverType, aopFolder, scriptFileName);
	
	//Build the message to display
	String result = getMessage(aopGenerationSuccess, copyAgentSuccess, scriptFileCopied, aopFolder, scriptFileName, serverType);
	out.println(result);
	
	return;
}
%>


<html lang="en">
<head>
<meta charset="utf-8">
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta name="description" content="">
<meta name="author" content="">
<title>MaintainJ - Reverse Engineer Java Like Never Before</title>
<meta name="description" content="MaintainJ, an Eclipse plug-in, generates UML sequence and class diagrams for a given use case, helping users to quickly understand a complex Java/J2EE application. MaintainJ logs runtime method execution trace and uses that to render sequence and class diagrams ">
<meta name="keywords" content="Automatic Sequence Diagram, Eclipse plugin, Generate Eclipse UML Diagram, Generate Eclipse Sequence Diagram, Debug Java, Reverse Engineer Sequence Diagram,Class Diagrams, Debug Sequence Diagram, Reverse Engineering">
<!-- Bootstrap Core CSS -->
<link href="css/bootstrap.min.css" rel="stylesheet">

<!-- Custom CSS -->
<link href="css/default.css" rel="stylesheet">

<!-- Custom Fonts -->
<link href="css/font-awesome.min.css" rel="stylesheet" type="text/css">

<!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
<!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
<!--[if lt IE 9]>
<script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
<script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
<![endif]-->

</head>






<body id="header">
<!-- Navigation -->
<nav class="navbar navbar-inverse" role="navigation">
  <div class="container">
    <!-- Brand and toggle get grouped for better mobile display -->
    <div class="navbar-header">
      <button type="button" class="navbar-toggle" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1"> <span class="sr-only">Toggle navigation</span> <span class="icon-bar"></span> <span class="icon-bar"></span> <span class="icon-bar"></span> </button>
      <a class="navbar-brand" href="index.jsp">MaintainJ</a> </div>
    <!-- Collect the nav links, forms, and other content for toggling -->
    <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
      <ul class="nav navbar-nav navbar-right">
        <li> <a href="index.jsp">Home</a> </li>
      </ul>
    </div>
    <!-- /.navbar-collapse -->
  </div>
  <!-- /.container -->
</nav>



<div id="configure">
  <section id="whitebg" class="wow fadeIn" style="display: block;">
    <div class="container">
		<div class="row">
			<h2>Install MaintainJ to capture the trace files on a JEE server</h2>
		</div>
	
		<div class="row">
			<div class="col-sm-12">
			  <h3>Step 1: Install and Configure MaintainJ </h3>
			  <p>
			  i) Copies MaintainJAgent.jar to a folder (on the same computer where JEE server is running)<br>
			  ii) Generates the maintainj.xml configuration file<br>
			  iii) Generates a startup script for your server (except for WebSphere)
			  </p>
			  <form name="aop">
			  	<p>
			  	*Server Type:			 			  	
		  	    <input type="radio" name="serverType" value="tomcat" >
		        Tomcat
		        <input type="radio" name="serverType" value="jboss" >
		        JBoss AS (Up to V6)
		        <input type="radio" name="serverType" value="wildfly">
		        JBoss AS 7+ or WildFly or JBoss EAP 6+		        
		        <input type="radio" name="serverType" value="weblogic">
		        WebLogic
		        <input type="radio" name="serverType" value="websphere">
		        WebSphere
		        <input type="radio" name="serverType" value="other">
		        Other
			  	<br>				
				*Package(s) to trace: <input type="text" required="required" name="incpackages" size="30" value=""> Example: com.package1,com.package2
				<br>
				(Enter the one or more package names in your application that you want to trace.
				Separate the package names by a comma. All classes in the given package(s) and
				their subpackages will be traced.)
				<br>							
				Package(s) NOT to trace:<input type="text" name="excpackages" size="30" value="">
				<br>
				(Enter any package names that you want to exclude from tracing.
				Separate the package names by a comma.)
				<br>		
				</p>
				<div id="databaseTypeId">
					<p>
					Database used by application				 
					<input type="radio" name="databaseType" value="Oracle"> Oracle 
					<input type="radio" name="databaseType" value="DB2"> DB2 
					<input type="radio" name="databaseType" value="MySQL"> MySQL
					<input type="radio" name="databaseType" value="SQL Server"> SQL Server 
					<input type="radio" name="databaseType" value="Sybase"> Sybase 
					<input type="radio" name="databaseType" value="PostgreSQL"> PostgreSQL 
					<input type="radio" name="databaseType" value="Derby"> Derby
				</p>
				</div>
				<p>							
				<input type="checkbox" name="logData"> Do not capture method call parameter data (Select only when MaintainJ makes running your application very slow. SQL calls also won't be captured)
				<br>				
				<input type="checkbox" name="verbose"> Show debug messages in console (System.err)
				<br>
				*Enter a folder to place the MaintainJ's files in: 
				<input type="text" name="aopFolder" value=""> Example: c:\maintainj
				<br>
				<input type="button" value="Install" 
					onclick="javascript:downloadAopXml(aop.serverType, aop.incpackages.value, aop.excpackages.value, 
							aop.databaseType, aop.verbose.checked, aop.logData.checked, aop.aopFolder.value)">
				<span id="generateAOPContent" style="display:none;"></span>
				</p>				
			  </form>

			</div>
		</div>

		<div id="step2" class="row">		
			<div class="col-sm-12">
			  <h3>Step 2: Verify Installation (after restarting the server with added JVM args)</h3>
			  	<p>
			  	<input type="button" value="Verify Installation" onclick="javascript:verifyInstallation()">
				<span id="verifyContent"></span>	
				</p>
			</div>
		</div>
  </section>
</div>






<script type="text/javascript">
var folderSelected = "";
function verifyInstallation(){
	$.get( "install.jsp?nextAction=verify", function( data ) {
		$( "#verifyContent" ).html( data );
	});	
}

function getServerSteps(serverType){
	$.get( "install.jsp?nextAction=serverSteps&serverType="+serverType+"&folderSelected="+folderSelected, function( data ) {
		$( "#serverStepsContent" ).html( data );
	});	
}

function getRadioSelectedValue(radioList){
	var selectedValue = "";
	for(var i = 0; i < radioList.length; i++) {
	   if(radioList[i].checked == true) {
		   selectedValue = radioList[i].value;
	   }
	 }	
	return selectedValue;
}

function downloadAopXml(serverTypeList, incpackages, excpackages, databaseTypeList, verbose, logData, aopFolder){
	//Get the serverType and databaseType values from the list
	serverType = getRadioSelectedValue(serverTypeList); 
	databaseType = getRadioSelectedValue(databaseTypeList);
	
	$("#generateAOPContent").fadeOut();
	
	//Set the global variable 
	folderSelected = aopFolder;
	
	errorMessage = "";
	if(serverType.trim() === ""){
		errorMessage = "Please choose the server type";
	} else if(incpackages.trim() === ""){
		errorMessage = "Package(s) to trace is mandatory";	
	} else if(aopFolder.trim() === ""){
		errorMessage = "Please enter the folder to place MaintainJ's files";	
	}  
	if(errorMessage != ""){
		errorMessage = "<span class=\"alert alert-danger\">" + errorMessage + "</span>";
		$( "#generateAOPContent" ).html( errorMessage );
		$("#generateAOPContent").fadeIn();
		return;
	}
	
	$.post("install.jsp", 
		{ 	nextAction:"downloadAopXml", serverType:serverType,  incpackages:incpackages,
			excpackages:excpackages,databaseType:databaseType, verbose:verbose, 
			logData:logData,aopFolder:aopFolder
		},
		function(data){
			$("#generateAOPContent" ).html( data );	
			$("#generateAOPContent").fadeIn();

			if (data.substring(0, 7) === "Failed") {
			    //Failed to generate aop.xml. Ignore 
			} else {
				//aop.xml generated successfully. Show the next step.
				$( "#aopFolder1" ).html( aopFolder );
				$( "#aopFolder2" ).html( aopFolder );					
			}
        	}
        );	
}
</script>

<script src="js/jquery.js"></script>
<!-- Bootstrap Core JavaScript -->
<script src="js/bootstrap.min.js"></script>
</body>
</html>