<!DOCTYPE html>

<%@page import="com.maintainj.aspect.JoinPointMap"%>
<%@page import="java.lang.management.RuntimeMXBean"%>
<%@page import="java.net.ServerSocket"%>
<%@page import="java.io.InputStreamReader"%>
<%@page import="java.io.BufferedWriter"%>
<%@page import="java.io.BufferedReader"%>
<%@page import="java.io.IOException"%>
<%@page import="java.io.FileOutputStream"%>
<%@page import="java.io.InputStream"%>
<%@page import="java.io.FileWriter"%>
<%@page import="java.io.OutputStream"%>
<%@page import="java.io.FileInputStream"%>
<%@page import="com.maintainj.aspect.AspectUtil"%>
<%@page import="java.lang.management.ManagementFactory"%>
<%@page import="java.lang.reflect.Method"%>
<%@page import="java.util.List"%>

<%!

Class getJoinPointMap(){
	Class jpMap = null;
	try{
		jpMap = Class.forName("com.maintainj.aspect.JoinPointMap");
	} catch (ClassNotFoundException cnfe){
		System.out.println("Fatal error. Did you verify the installation of MaintainJ? MaintainJAspect.jar should be in classpath.");
	}
	return jpMap;
}

/**
 * Sets if join points should be added to this list
 * @param addJoinPoint set to true if join points should be added 
 * to list. Set to false otherwise.
 */
void setAddJoinPoint(boolean addJp) {
	Class jpMap =  getJoinPointMap();
	if(jpMap == null){
		return;
	}
	try{
		Method method = jpMap.getMethod("setAddJoinPoint", new Class[]{boolean.class});
		method.invoke(null, new Object[]{new Boolean(addJp)});
	} catch (Exception e){
		System.out.println("Fatal error. Error calling JoinPointMapReflector.setAddJoinPoint(boolean)");
	}
}			
	
/**
 * Writes the join points of the use case to a file. One trace file 
 * for each thread in the use case is generated. One trace file 
 * with all the calls in all threads also is generated with name 
 * _AllThreads.ser appended to the given file name. 
 * Returns an String[][] where String[][0] is the ser file name
 * and String[][1] is the size of the jp list. 
 * @param fileName name of the trace file. Do not give any extension.
 * The extension .ser is automatically added to the file name.
 */	
String[][] writeToFile(String fileName, boolean doNotMerge) throws Exception{
	//Instead of calling JoinPointMap directly, using reflection to avoid compilation error/warning in eclipse
	//return JoinPointMap.writeToFile(fileName);
	
	String[][] traceFiles = new String[0][0];
	Class jpMap =  getJoinPointMap();
	if(jpMap == null){
		return traceFiles;
	}
	
	Method method = jpMap.getMethod("writeToFile", new Class[]{String.class, boolean.class});
	traceFiles = (String[][])method.invoke(null, new Object[]{fileName, new Boolean(!doNotMerge)});

	return traceFiles;
}

/**
 * Returns true if there is no trace information.
 * @return
 */
boolean isEmpty(){
	boolean isEmpty = true;
	Class jpMap =  getJoinPointMap();
	if(jpMap == null){
		return isEmpty;
	}
	try{
		Method method = jpMap.getMethod("isEmpty", new Class[]{});
		Object isEmptyObj = (Object)method.invoke(null, null);
		isEmpty = ((Boolean)isEmptyObj).booleanValue();
	} catch (Exception e){
		System.out.println("Fatal error. Error calling JoinPointMapReflector.isEmpty()");
	}	
	
	return isEmpty;
}

/**
 * Clears all lists and sets the jpHashcode to zero.
 * Call this method after writing the call lists to trace file. 
 *
 */
void clear(){
	Class jpMap =  getJoinPointMap();
	if(jpMap == null){
		return;
	}
	try{
		Method method = jpMap.getMethod("clear", new Class[]{});
		method.invoke(null, null);
	} catch (Exception e){
		System.out.println("Fatal error. Error calling JoinPointMapReflector.clear()");
	}			
}

boolean isNodeAgentStarted(ServletContext servletContext){
	Object nodeAgentStarted = servletContext.getAttribute("nodeAgentStarted");
	boolean isNodeAgentStarted = (nodeAgentStarted!=null && ((String)nodeAgentStarted).length() > 0);
	return isNodeAgentStarted;
}

final String  NODE_AGENT_PORT_VM_ARG = "com.maintainj.agent.port";

String getNodeAgentPortInJVMArgs(){		
	String nodeAgentPort = System.getProperty(NODE_AGENT_PORT_VM_ARG);
	if(nodeAgentPort != null && nodeAgentPort.length() > 0){
		return nodeAgentPort;
	} else {
		return null;
	}
}

/**
 * Starts the node agent AND sets the nodeAgentStarted in ServletContext 
 * with the node agent port value
 * @param servletContext
 * @param agentPort
 */
void startNodeAgent(ServletContext servletContext, String agentPort){
	startNodeAgent(agentPort);
	//Set the port number in ServletContext
	servletContext.setAttribute("nodeAgentStarted", agentPort);
}

/**
 * Call this only with a valid agent port
 * @param agentPort
 */
void startNodeAgent(String agentPort){
	int portNumber = 0;
	try{
		portNumber = Integer.parseInt(agentPort);
	} catch (Exception e){
		//Ignore
	}
	AspectUtil.startNodeAgent(portNumber);
}

boolean isAgentPortValid(String portString){
	boolean valid = true;
	int port = -1;
	try{
		port = Integer.parseInt(portString);
	} catch (Exception exp){
		valid = false;
	}
	if(valid && port > 0){
		ServerSocket agentSocket = null;
		try{
			agentSocket = new ServerSocket(port);
		} catch (Exception exp){
			valid = false;																			
		} finally {
			if(agentSocket != null){
				try{agentSocket.close();} catch(Exception exp){}
			}
		}
	}				
	return valid;
}

boolean verifyInstallation(){
	boolean installationSuccess = false;
	try{
		boolean mnjJarInCP = false;
		boolean aspectRegistered = false;
		Class.forName("com.maintainj.aspect.JoinPointMap");
		mnjJarInCP = true;
		Class.forName("com.maintainj.inst.J2EEAspect");
		aspectRegistered = true;
		installationSuccess = (mnjJarInCP && aspectRegistered);	
	} catch (Throwable th){
		th.printStackTrace();
	}
	return installationSuccess;
}
%>

<%

//Trace file names and sizes returned from JoinPointMap.writeToFile
String[][] filesAndSizes = null;
boolean exceptionWritingTraceFiles = false;

boolean noJpsCaptured = false;
//Check if either start or stop is clicked
String nextAction = request.getParameter("nextAction");
boolean isNextActionEmpty = (nextAction == null || nextAction.length() == 0);
if(!isNextActionEmpty && "start".equals(nextAction)){			
	//Start clicked. Turn on tracing
	setAddJoinPoint(true);
} else if(!isNextActionEmpty && nextAction.equals("stop")){

System.out.println(request.getParameter("donotmerge"));
	//Stop clicked. Turn off tracing.
	setAddJoinPoint(false);
	String fileName = request.getParameter("fileName");	
	if(isEmpty()){
		noJpsCaptured = true;
	} else {
		try{
			String donotmerge = request.getParameter("donotmerge");
			filesAndSizes = writeToFile(fileName, "on".equals(donotmerge));
		} catch (Exception e){
			exceptionWritingTraceFiles = true;
		} finally{
			clear();
		}
	}
	session.setAttribute("mnj.fileName", fileName);
}

//Node agent related logic
//Check if node agent is already started
boolean isNodeAgentStarted = isNodeAgentStarted(application);

//If server is started in Eclipse and node agent is not started yet, check if 
//agent port is specified in JVM args. If so, start node agent. 
if(!isNodeAgentStarted){
	String agentPort = getNodeAgentPortInJVMArgs();
	if(agentPort != null){
		//There is also no need to validate the port as it is validated 
		//before launching the server.
		startNodeAgent(application, agentPort);
		isNodeAgentStarted = true;
	}
}

//If node agent port is entered and 'Start Node Agent' is clicked, read the 
//port value, validate the port number and start the agent
String nodeAgentPort = request.getParameter("nodeAgentPort");
boolean startAgentClicked = false;
boolean isValidPort = false;

if(isNodeAgentStarted || nodeAgentPort == null || nodeAgentPort.length() == 0){
	//Ignore
} else {
	//Node agent port is entered by user. Javascript stops empty port number. 
	//The presence of port number in request indicates that 'Start Node Agent'
	//is clicked.
	startAgentClicked = true;
	//Validate port
	isValidPort = isAgentPortValid(nodeAgentPort);
	if(isValidPort){
		startNodeAgent(application, nodeAgentPort);
		isNodeAgentStarted = true;
	}
}

%>


<html lang="en">
<head>
<meta charset="utf-8">
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta name="description" content="">
<meta name="author" content="">
<title>MaintainJ - Reverse Engineer Java Like Never Before</title>
<meta name="description" content="MaintainJ, an Eclipse plug-in, generates UML sequence and class diagrams for a given use case, helping users to quickly understand a complex Java/J2EE application. MaintainJ logs runtime method execution trace and uses that to render sequence and class diagrams ">
<meta name="keywords" content="Automatic Sequence Diagram, Eclipse plugin, Generate Eclipse UML Diagram, Generate Eclipse Sequence Diagram, Debug Java, Reverse Engineer Sequence Diagram,Class Diagrams, Debug Sequence Diagram, Reverse Engineering">
<!-- Bootstrap Core CSS -->
<link href="css/bootstrap.min.css" rel="stylesheet">

<!-- Custom CSS -->
<link href="css/default.css" rel="stylesheet">

<!-- Custom Fonts -->
<link href="css/font-awesome.min.css" rel="stylesheet" type="text/css">

<!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
<!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
<!--[if lt IE 9]>
<script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
<script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
<![endif]-->



</head>






<body id="header">
<!-- Navigation -->
<nav class="navbar navbar-inverse" role="navigation">
  <div class="container">
    <!-- Brand and toggle get grouped for better mobile display -->
    <div class="navbar-header">
      <button type="button" class="navbar-toggle" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1"> <span class="sr-only">Toggle navigation</span> <span class="icon-bar"></span> <span class="icon-bar"></span> <span class="icon-bar"></span> </button>
      <a class="navbar-brand" href="index.jsp">MaintainJ</a> </div>
    <!-- Collect the nav links, forms, and other content for toggling -->
    <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
      <ul class="nav navbar-nav navbar-right">
        <li> <a href="index.jsp">Home</a> </li>
      </ul>
    </div>
    <!-- /.navbar-collapse -->
  </div>
  <!-- /.container -->
</nav>



<div id="configure">
  <section id="whitebg" class="wow fadeIn" style="display: block;">
    <div class="container">
    
    	<% if(!verifyInstallation()) {%>
  				<h2>MaintainJ is not properly installed</h2>
  				<p>Please ensure that  MaintainJ installation is successful in step 2 of <a href="install.jsp">MaintainJ Installation</a>.
  			<%
  			return;
  		} %>
	    <% if(noJpsCaptured){ %>
			<h2>Call trace is empty</h2>
			
			<p> 
			Between Start Tracing and Stop Tracing, did you run a use case that 
			uses the classes under the package(s) under instrumentation scope? 
			If you did, ensure that MaintainJ installation is successful in step 3 of 
			<a href="install.jsp">MaintainJ Installation</a>.
		<%} else if(exceptionWritingTraceFiles){%>
			<p>Trace files could not be written to folder <%=request.getParameter("fileName")%>.
			Please check if the folder is valid and has write permission. 
		<%} else if(filesAndSizes != null && filesAndSizes.length > 0){%>
	        <h2>Trace File(s) Generated</h2>
			<p>The following trace files are generated. Copy them to a 
			MaintainJ Trace Files project and open to view the sequence 
			and class diagrams. To create MaintainJ Trace Files project in Eclipse, click 
			File-&gt;New-&gt;Project-&gt;MaintainJ-&gt;MaintainJ Trace Files Project.
			<table border="1" width="85%" cellspacing="0" bordercolor="#C0C0C0" cellpadding="0">
				<tr>
	    			<td width="70%"><b>Trace File Name</b></td>
	    			<td width="30%" align="center"><b>Number of Calls</b></td>
	  			</tr>
			
				<% for(int i=0; i<filesAndSizes.length; i++){ %>
	  			<tr>
	    			<td width="80%"><%= filesAndSizes[i][0] %>&nbsp;</td>
	    			<td width="20%" align="center"><%= filesAndSizes[i][1] %>&nbsp;</td>
	  			</tr>
	  			<% } %>
			</table>		
			Note: If you do not find the above trace files in the folder, please check if the 
			given folder has write permission. You should also see the exception stack trace 
			to that effect in System.out.
			<hr color="#0000FF">
		<%} %>
    
		<h2>Capture Call Trace</h2>		
		<div class="row">
			<div class="col-sm-12">
			  <form name="startStopForm">
			  	<input type="hidden" name="nextAction" value="<%= (nextAction==null?"":nextAction) %>" />				

				<%if(isNodeAgentStarted) {%>
					MaintainJ Node Agent started successfully on port <%=application.getAttribute("nodeAgentStarted") %>. 
					Use the <a href="http://maintainj.com/userGuide.jsp?param=runtime#multivms">MaintainJ Node Manager</a>
					 to capture trace files on multiple JVMs. If you want to trace the applications just on this 
					 JVM, restart the server and reload this page.
				<%} else if("start".equals(nextAction)) { %>
					<p>From another browser window, perform an action in your application 
					for which you want to generate the diagrams.
					<p>Enter the trace file name (with absolute path and no extension) 
					and click Stop Tracing button. Tracing is turned off and call trace 
					is written to the given file. <p>
					<b>Trace File</b> 
					<%
						String mnjFileName = (String)session.getAttribute("mnj.fileName");
						if(mnjFileName == null || mnjFileName.length() == 0){
							mnjFileName = "c:\\temp\\CallTrace";
						}
					%>
					<INPUT type="text" name="fileName" size="55" value="<%=mnjFileName%>">		
					<INPUT type="button" name="stop" value="Stop Tracing" onClick="javascript:submitPage('stop')">
					<br>
					<input type="checkbox" name="donotmerge"> Do not merge trace files from different threads<br>
					(Choose when server is running low on memory. Trace files can be merged offline later in eclipse 
					using the wizard at File->New->Other->MaintainJ->Merge Trace Files)
				<%} else {
					if(isNextActionEmpty) {%>
						<h3>If you are tracing an application on a single JVM</h3>
					<% }
					if(isNextActionEmpty || "stop".equals(nextAction)) {%>
						<p>Click Start Tracing button to turn on tracing. Once tracing is turned on, 
						call trace will be logged for any action performed in the application.
						<p>
						<INPUT type="button" name="start" value="Start Tracing" onClick="javascript:submitPage('start')">	
						<p>
					<%}
					if(isNextActionEmpty) {%>
						<h3>If you are tracing applications on multiple JVMs</h3>
						<p>For example when a J2EE application is caling a web service on another JVM 
						and you want to see end-to-end call flow in a sequence diagram, use this.<p>
						Enter a valid port number to start the node agent.
						<%if(startAgentClicked && !isValidPort){ %>	
							<br><font color="#FF0000">Failed to start MaintainJ node agent. 
							Enter a valid port number which is not already in use.</font>
						<%} %>
						<br>
						The port for MaintainJ node agent: <input type="text" name="nodeAgentPort" size="5" maxlength="5" value='<%=nodeAgentPort==null?"":nodeAgentPort%>'/>
						<INPUT type="button" name="startNodeAgent" value="Start Node Agent" onClick="javascript:startNodeAgentSubmit();" />	
					<%} 
				}%>		
						<h3>If you need to reconfigure MaintainJ (to change the packages to trace)</h3>
						<a href="install.jsp">Reconfigure MaintainJ</a>
			  </form>
			</div>
		</div>

		</div>
  </section>
</div>

<script type="text/javascript">
	function submitPage(param){					
		if(param == 'stop' && document.startStopForm.fileName.value != null){
			var traceFileName = document.startStopForm.fileName.value;
			if(traceFileName == null || traceFileName== ""){
				alert("Please enter the trace file name.");
				return;
			}
		}
		document.startStopForm.nextAction.value = param;
		document.startStopForm.nextAction.action = "/maintainj/aspect.jsp";
		document.startStopForm.submit();
	}
	
	function startNodeAgentSubmit(){
		var agentPort = document.startStopForm.nodeAgentPort.value;
		if(agentPort == null || agentPort== ""){
			alert("Please enter a port number to start the node agent.");
			return;
		} else {
			document.startStopForm.submit();
		}
	}		
</script>

<script src="js/jquery.js"></script>
<!-- Bootstrap Core JavaScript -->
<script src="js/bootstrap.min.js"></script>
</body>
</html>